using System;
using System.Collections.Generic;
using System.Text;
using ceometric.ComputationalGeometry;
using ceometric.VectorGeometry;

//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
// You must reference both V# and G# first to run this program
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

namespace DelaunayTriangulation
{
    /// <summary>
    /// Pure point triangulation of real xyz terrain data.
    /// </summary>
    class Program
    {
        static void Main(string[] args)
        {
            //---------------------------------------------------------------------
            //Read some points from a sample point set in xyz format.
            //SPECIFY THE CORRECT FILE FORMAT HERE:
            //terrain_en.xyz has comma (",") decimal seperator
            //terrain_us.xyz has point (".") decimal seperator
            PointSet vertices = new PointSet();
            vertices.ReadFromFile(@"..\..\terrain_us.xyz", ';');
            //vertices.ReadFromFile(@"..\..\terrain_en.xyz", ';');

            //---------------------------------------------------------------------
            //Triangulate the pure xyz point data. Measure time
            KernelTimeGauge tg = new KernelTimeGauge();
            tg.Start();
            DelaunayTriangulation2d dt = new DelaunayTriangulation2d();
            dt.TriangulationPoints = vertices;
            dt.Triangulate();
            tg.Stop();
            List<Triangle> triangles = dt.GetTriangles(); //Get the triangles of the surface

            // Some statistics
            Console.WriteLine("Number of vertices : " + vertices.Count);
            Console.WriteLine("Number of triangles: " + triangles.Count);
            Console.WriteLine("Triangulation time : " + tg.DurationInMilliSeconds + "ms");

            //---------------------------------------------------------------------
            // Visualize in dxf format
            // Get the free DWG TrueView viewer at www.autodesk.com to view dxf
            Scene sc = new Scene();
            Scene.Layer pLayer = new Scene.Layer("Vertices", 7);
            Scene.Layer tLayer = new Scene.Layer("Delaunay_Triangles", 3);

            // Add objects to the scene
            sc.Add(vertices, pLayer);
            sc.Add(triangles, tLayer);

            // Output the scene as dxf
            string outfile = @"e:\terrain.dxf";
            sc.WriteDxf(outfile);
            Console.WriteLine("dxf written to " + outfile);

            Console.WriteLine("Hit any key to continue.");
            Console.ReadLine();
        }
    }
}